
function std_cc=std_gmm_corr
% This function computes the standard error of the correlation coefficient
% of two series.  It calls functions gmm_corr.m, corrfun.m and
% gmm1_J_corr.m
% Written by Pedro Silos


global Ndata data weight m1 m2 m12 m11 m22 dimv rhos detind

z = corrcoef(data);
rhos = z(1,2);

dimx = 5; % dimension of the vector of parameters. 
dimv = 5; % dimension of the vector of moments
weight = eye(dimv);
u = zeros(Ndata,dimv);
 
x0 = 0.5*ones(dimx,1);
 
for i=1:10
x0 = fminsearch('gmm1',x0);
u(:,1) = data(:,1) - x0(1);
u(:,2) = data(:,2) - x0(2);
u(:,3) = data(:,1).*data(:,2) - x0(3);
u(:,4) = data(:,1).^2-x0(4);
u(:,5) = data(:,2).^2-x0(5);
weight = inv((1/Ndata)*(u'*u));
end

% Asymptotic variance;

% compute jacobian (matrix of partial derivatives of gmm1 wrt parameters)
% 
 J = zeros(dimv,dimx);
 eps = 1d-8;
% 
 for i=1:dimx
 evec = zeros(dimx,1);
 evec(i) = eps;
 x=x0+evec;
 J(:,i) = ((gmm1_J(x)-gmm1_J(x0))./eps)';
 end
 
 
 % The asymptotic covariance matrix is:
 
 COVAR = inv(J'*weight*J)/Ndata;
 
 COVAR(1,1);
 std(data(:,1))^2/Ndata;
 
 x = x0;
 
 % Now compute the asymptotic covariance of the correlation coefficient.
 % Compute the gradient of the correlation coefficient with respect to 
 % the parameters. 
 
 grad_cf = zeros(dimx,1);
 
 for i=1:dimx;
     evec = zeros(dimx,1);
     evec(i) = eps;
     grad_cf(i) = (corrfun(x0+evec)-corrfun(x0))/eps;
 end
 
x = x0;
% exact gradient
Den = (sqrt(x(4)-x(1)^2)*sqrt(x(5)-x(2)^2));
g(1) = (-x(2)*Den-(x(3)-x(1)*x(2))*0.5*((x(4)-x(1)^2)*(x(5)-x(2)^2))^(-0.5)*(-2*x(1))*(x(5)-x(2)^2))/Den^2;
g(2) = (-x(1)*Den-(x(3)-x(1)*x(2))*0.5*((x(4)-x(1)^2)*(x(5)-x(2)^2))^(-0.5)*(-2*x(2))*(x(4)-x(1)^2))/Den^2;
g(3) = 1/Den;
g(4) = (-(x(3)-x(1)*x(2))*0.5*((x(4)-x(1)^2)*(x(5)-x(2)^2))^(-0.5)*(x(5)-x(2)^2))/Den^2;
g(5) = (-(x(3)-x(1)*x(2))*0.5*((x(4)-x(1)^2)*(x(5)-x(2)^2))^(-0.5)*(x(4)-x(1)^2))/Den^2;

  
 
% Variance of correlation coefficient:
var_cc = (grad_cf'*inv(J'*weight*J)*grad_cf)/Ndata;
disp('Correlation coefficient is:')
disp(rhos)
% Standard deviation of correlation coefficient:
std_cc = sqrt(var_cc);
disp('Std. Error of the Correlation Coefficient is:');
 